/**
 * @author James Tomkinson
 * @copyright StreamBase Systems, Inc. 2012
 */
package com.streambase.services.adapters.webservice;


import java.net.URL;
import java.util.ArrayList;

import org.slf4j.Logger;
import com.streambase.services.webservicesclients.JerseyClient1;

import com.streambase.sb.*;
import com.streambase.sb.Schema.Field;
import com.streambase.sb.operator.*;

/**
 * Generated by JDT StreamBase Client Templates (Version: 7.1.8.1202221217).
 *
 */
public class WeatherRestfulWebServiceAdapter extends Operator implements
		Parameterizable {

	public static final long serialVersionUID = 1330467211649L;
	private Logger logger;
	// Properties
	private String URL;
	private String displayName = "RESTful Web Service";
	// Local variables
	private int inputPorts = 1;
	private int nextOutputPort = 0;
	private Schema outputSchema; // caches the Schemas given during typecheck for use at processTuple
	private static final String BASE_URI = 
			"http://graphical.weather.gov/xml/sample_products/browser_interface/ndfdXMLclient.php?product=time-series&begin=2004-01-01T00:00:00&end=2013-04-21T00:00:00&maxt=maxt&mint=mint&pop12=pop12&zipCodeList=";
	private JerseyClient1 client;

	/**
	 * The constructor is called when the Operator instance is created
	 */
	public WeatherRestfulWebServiceAdapter() {
		super();
		logger = getLogger();
		setPortHints(1, 1);
		setDisplayName(displayName);
		setShortDisplayName(this.getClass().getSimpleName());
		setURL( BASE_URI );
	}

	/**
	 * The typecheck method is called after the Operator instance is connected in the StreamBase
	 * application, allowing the Operator to validate its properties. 
	 */
	public void typecheck() throws TypecheckException {
		// Typecheck: require a specific number of input ports
		requireInputPortCount(inputPorts);

		// Note that we will not check the input schema, as this will only trigger the service (allow no field input stream)
		requireInputPortCount(1);
		
		ArrayList<Field> outFields = new ArrayList<Field>();
		outFields.add(new Field("message", CompleteDataType.forString()));
		outputSchema = new Schema( null, outFields );
		setOutputSchema(0, outputSchema);
	}

	/**
	 * This method will be called by the StreamBase server for each Tuple given
	 * to the Operator to process. This is the only time an operator should 
	 * enqueue output Tuples.
	 */
	public void processTuple(int inputPort, Tuple tuple)
			throws StreamBaseException {
		if (logger.isInfoEnabled()) {
			logger.info("operator processing a tuple at input port" + inputPort);
		}
		// Only the first input port is processed; see the template code for typecheck()
		if (inputPort > 0) {
			logger.info("operator skipping tuple at input port" + inputPort);
			return;
		}

		// Create a new output tuple from the Schema at the port we are about to send to
		Tuple out = getOutputSchema(nextOutputPort).createTuple();
		String zipcode = null;
		if( !tuple.isNull("zipcode"))
			zipcode = tuple.getString("zipcode");

		// Get the result from the web service client call
		String result = client.getWeatherForZipCode( URL, zipcode );
		
		// Output the result to the StreamBase application
		out.setString(0, result );
		sendOutput(0, out);
	}

	/**
	 * If typecheck succeeds, the init method is called before the StreamBase application
	 * is started. 
	 */
	public void init() throws StreamBaseException {
		client = new JerseyClient1( BASE_URI );
		super.init();
	}

	/**
	 *  The shutdown method is called when the StreamBase server is in the process of shutting down.
	 */
	public void shutdown() {

	}

	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	public void setURL(String URL) {
		this.URL = URL;
	}

	public String getURL() {
		return this.URL;
	}
	
	
	@Override
	public URL getIconResource(IconKind iconType){
		switch(iconType){
			case CANVAS_OVERLAY_ICON:
				return getClass().getResource("weather24.png");
			case PALETTE_ICON_LARGE:
				return getClass().getResource("weather32.png");
			case PALETTE_ICON_SMALL:
				return getClass().getResource("weather16.png");
		}
		return null;
	}
	
}
